/* -*-c-*-
 *
 * $Id: prim.c,v 1.1 1999/12/22 15:58:59 mdw Exp $
 *
 * Finding primitive elements
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: prim.c,v $
 * Revision 1.1  1999/12/22 15:58:59  mdw
 * Search for primitive elements using prime-search equipment.
 *
 */

/*----- Header files ------------------------------------------------------*/

#include "mp.h"
#include "mpint.h"
#include "mpmont.h"
#include "mprand.h"
#include "pgen.h"
#include "prim.h"

/*----- Main code ---------------------------------------------------------*/

/* --- @prim_test@ --- */

int prim_test(int rq, pgen_event *ev, void *p)
{
  prim_ctx *c = p;
  int rc = rq;

  switch (rq) {
    case PGEN_BEGIN:
      return (PGEN_TRY);
    case PGEN_TRY: {
      mp *x = MP_NEW;
      mp *f = c->f;
      rc = PGEN_FAIL;

      x = mpmont_exp(&c->mm, x, ev->m, f);
      if (MP_CMP(x, ==, MP_ONE))
	goto done;
      if (c->n == 0) {
	mp_drop(ev->m);
	ev->m = MP_COPY(x);
      } else {
	size_t n = c->n - 1;
	f++;
	while (n) {
	  x = mpmont_exp(&c->mm, x, ev->m, f);
	  if (MP_CMP(x, ==, MP_ONE))
	    goto done;
	  n--; f++;
	}
      }
      rc = PGEN_DONE;
    done:
      mp_drop(x);
    } break;
  }

  return (rc);
}

/* --- Trivial stepping functions -----------------------------------------*/

/* --- @prim_step@ --- */

int prim_step(int rq, pgen_event *ev, void *p)
{
  unsigned *i = p;
  switch (rq) {
    case PGEN_BEGIN:
    case PGEN_TRY:
      if (*i >= NPRIME)
	return PGEN_FAIL;
      ev->m = mp_fromint(ev->m, primetab[(*i)++]);
      return (PGEN_TRY);
  }
  return (0);
}

/*----- That's all, folks -------------------------------------------------*/
